# R code to calculate nonspatial and spatial phylogenetic diversity area curves for an ecological data set. The code assumes that the samp matrix, the object
# that contains the samples/plots/communities with species as columns and samples as rows, is ordered such that consecutive samples in the matrix
# are spatially closer to each other than to non-consecutive samples.


library(picante)

pdac.psv<-function(samp,tree, reps=100, lt=1)
{
  #reps  number of time to calculate mean values for each spatial scale
  Y <- samp   #matrix of samples
  Y[Y>0] <- 1
  m <- dim(Y)[1] #number of samples
  
  SAcurve<-NULL
  for(ss in 1:2)
  {
    for(a in 1:round(lt * m))
    {
      ratio <- NULL
      for(tt in 1:reps)
      {
        if(ss==1)
        {                  #nonspatial curve
          pick <- sample(m)
          pick <- pick[1:a]
        } else {           #spatial curve
          pick <- sample(m+1-a)[1]
          pick <- pick:(pick+a-1)
        }
          if(a==1) { y<-Y[pick,]} else {y <- colSums(Y[pick,])>0}
          ratio <- rbind(ratio,psv(y,tree,compute.var=FALSE))
      }
      SAcurve <- rbind(SAcurve,c(ss,a,mean(ratio[,1],na.rm=TRUE),sd(ratio[,1],na.rm=TRUE),mean(ratio[,2]),sd(ratio[,2])))
    }
  }
  SAcurve<-data.frame(SAcurve)
  SAcurve[SAcurve[,1]==1,1]<-"nonspatial"
  SAcurve[SAcurve[,1]=="2",1]<-"spatial"

  colnames(SAcurve)<-c("curve","aggregate","mean.psv","sd.psv","mean.sr","sd.sr")
  return(SAcurve)
}

#example
library(picante)

pdac <- pdac.psv(phylocom$samp,phylocom$phylo,reps=30,lt=1)

par(las=1)
plot(pdac[,2:3],type="n",ylab="phylogenetic diversity (PSV)",xlab="area (# of samples)")
lines(pdac[pdac[,1]=="nonspatial",2:3],col="red")
points(pdac[pdac[,1]=="nonspatial",2:3],col="red",pch=19)
lines(pdac[pdac[,1]=="spatial",2:3],col="blue")
points(pdac[pdac[,1]=="spatial",2:3],col="blue",pch=19)

legend(x=4,y=0.78,pch=19,legend=c("nonspatial","spatial"),lty=1,col = c("red","blue")) 

